@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: buildhash.c,v 1.76 2020-12-30 22:33:41-08 geoff Exp $";
+    "$Id: buildhash.c,v 1.77 2023-10-30 13:29:43-07 geoff Exp $";
 #endif
 
 #define MAIN
@@ -47,6 +47,10 @@ static char Rcs_Id[] =
 
 /*
  * $Log: buildhash.c,v $
+ * Revision 1.77  2023-10-30 13:29:43-07  geoff
+ * Fix a place where buildhash could reference beyond the end of the hash
+ * table, causing a segfault if that happened to cross a page boundary.
+ *
  * Revision 1.76  2020-12-30 22:33:41-08  geoff
  * Add the character-set options to the hash file.
  *
@@ -447,8 +451,8 @@ static void filltable ()
 		dp->next = freepointer;
 		dp = freepointer;
 
-		while ((freepointer->flagfield & USED)
-		  &&  freepointer < hashend)
+		while (freepointer < hashend
+		  &&  (freepointer->flagfield & USED))
 		    freepointer++;
 		}
 	    }
