@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: defmt.c,v 1.64 2015-02-08 16:09:24-08 geoff Exp $";
+    "$Id: defmt.c,v 1.65 2020-12-30 22:33:41-08 geoff Exp $";
 #endif
 
 /*
@@ -52,6 +52,9 @@ static char Rcs_Id[] =
 
 /*
  * $Log: defmt.c,v $
+ * Revision 1.65  2020-12-30 22:33:41-08  geoff
+ * Support SQUEEZE_STRINGS.
+ *
  * Revision 1.64  2015-02-08 16:09:24-08  geoff
  * When skipping curly-braced environment arguments, allow nested curlies.
  *
@@ -239,6 +242,7 @@ static int	tagcmp P ((unsigned char ** a, unsigned char ** b));
 			((c) == TEXPERCENT))
 
 static int	    TeX_comment = 0;
+static int	    wordadjust;         /* Stringchar difference in word len */
 
 /*
  * The following variables are used to save the parsing state when
@@ -510,7 +514,7 @@ static unsigned char * skiptoword (bufp) /* Skip to beginning of a word */
 			bufp += 2;
 			if (*bufp == '+'  ||  *bufp == '-')
 			    bufp++;
-			/* This looks wierd 'cause we
+			/* This looks weird 'cause we
 			** assume *bufp is now a digit.
 			*/
 			bufp++;
@@ -565,6 +569,12 @@ static unsigned char * skiptoword (bufp) /* Skip to beginning of a word */
     return bufp;
     }
 
+/*
+ * Return a pointer to the end of a word.  As a side effect, sets
+ * wordadjust to reflect the difference between the true length of the
+ * word in bytes and its apparent length in characters; this is used
+ * for the SQUEEZE_STRINGS option.
+ */
 unsigned char * skipoverword (bufp) /* Return pointer to end of a word */
     register unsigned char *
 			bufp;	/* Start of word -- MUST BE A REAL START */
@@ -574,6 +584,7 @@ unsigned char * skipoverword (bufp) /* Return pointer to end of a word */
     register int	scharlen; /* Length of a string character */
 
     lastboundary = NULL;
+    wordadjust = 0;
     for (  ;  ;  )
 	{
 	if (*bufp == '\0')
@@ -590,6 +601,8 @@ unsigned char * skipoverword (bufp) /* Return pointer to end of a word */
 	    {
 	    bufp += scharlen;
 	    lastboundary = NULL;
+            if (chartypes[defstringgroup].options & SQUEEZE_STRINGS)
+                wordadjust += scharlen - 1;
 	    }
 	/*
 	** Note that we get here if a character satisfies
@@ -623,6 +636,7 @@ unsigned char * skipoverword (bufp) /* Return pointer to end of a word */
 void checkline (ofile)
     FILE *		ofile;
     {
+    unsigned char *	currentchar;	/* Location in contextbufs */
     register unsigned char *
 			p;
     register unsigned char *
@@ -631,6 +645,7 @@ void checkline (ofile)
     register int	len;
     register int	i;
     int			ilen;
+    unsigned char *	wordstart;      /* Where current word started */
 
     currentchar = filteredbuf;
     len = strlen ((char *) filteredbuf) - 1;
@@ -693,11 +708,11 @@ void checkline (ofile)
 	    }
 	}
 	
-    for (  ;  ;  )
+    for (  ;  ;  contextoffset -= wordadjust)
 	{
-	p = skiptoword (currentchar);
-	if (p != currentchar)
-	    copyout (&currentchar, p - currentchar);
+	wordstart = skiptoword (currentchar);
+	if (wordstart != currentchar)
+	    copyout (&currentchar, wordstart - currentchar);
 
 	if (*currentchar == 0)
 	    break;
@@ -825,8 +840,7 @@ void checkline (ofile)
 			      easypossibilities ? '&' : '?',
 			      (char *) ctoken,
 			      easypossibilities,
-			      (int) ((currentchar - filteredbuf)
-				- strlen ((char *) ctoken)) + contextoffset);
+			      (int) (wordstart - filteredbuf + contextoffset));
 			for (i = 0;  i < MAXPOSSIBLE;  i++)
 			    {
 			    if (possibilities[i][0] == 0)
@@ -846,8 +860,7 @@ void checkline (ofile)
 			else
 			    (void) fprintf (ofile, "# %s %d\n",
 			      (char *) ctoken,
-			      (int) ((currentchar - filteredbuf)
-				- strlen ((char *) ctoken)) + contextoffset);
+			      (int) (wordstart - filteredbuf + contextoffset));
 			}
 		    }
 		}
