@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: dump.c,v 1.18 2001/07/25 21:51:47 geoff Exp $";
+    "$Id: dump.c,v 1.19 2005/04/14 14:38:23 geoff Exp $";
 #endif
 
 /*
@@ -25,16 +25,8 @@ static char Rcs_Id[] =
  *    such.  Binary redistributions based on modified source code
  *    must be clearly marked as modified versions in the documentation
  *    and/or other materials provided with the distribution.
- * 4. Any web site or other electronic service that offers ispell for
- *    download or other electronic transfer as a separate entity, in
- *    either source or binary form, must also include a prominent statement
- *    indicating that information about ispell can be obtained from the
- *    following Web site URL:
- *	http://fmg-www.cs.ucla.edu/geoff/ispell.html
- *    If the offering service supports hyperlinks, the aforementioned
- *    Web site must also be offered as a hyperlink.  Condition #4 does
- *    not apply if ispell is offered only as part of a larger, aggregated
- *    product such as a word processor or packaged operating system.
+ * 4. The code that causes the 'ispell -v' command to display a prominent
+ *    link to the official ispell Web site may not be removed.
  * 5. The name of Geoff Kuenning may not be used to endorse or promote
  *    products derived from this software without specific prior
  *    written permission.
@@ -53,7 +45,6 @@ static char Rcs_Id[] =
  */
 
 /*
- * $Log: dump.c,v $
  * Revision 1.18  2001/07/25 21:51:47  geoff
  * Minor license update.
  *
