@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: ispell.c,v 1.161 2005/05/25 14:13:53 geoff Exp $";
+    "$Id: ispell.c,v 1.163 2022-02-06 00:55:29-08 geoff Exp $";
 #endif
 
 #define MAIN
@@ -47,7 +47,16 @@ static char Rcs_Id[] =
 
 /*
  * $Log: ispell.c,v $
- * Revision 1.161  2005/05/25 14:13:53  geoff
+ * Revision 1.163  2022-02-06 00:55:29-08  geoff
+ * Fix a small mistake in the previous change
+ *
+ * Revision 1.162  2022-02-06 00:26:32-08  geoff
+ * Don't process the -k keyword until the dictionary has been read, since
+ * -k uses chupcase and that function depends on the contents of the
+ * dictionary header.  This change also involves a rewrite of how
+ * arguments are processed; the code no longer modifies argc and argv.
+ *
+ * Revision 1.161  2005-05-25 07:13:53-07  geoff
  * Report the value of EXEEXT in "ispell -vv".
  *
  * Revision 1.160  2005/04/28 14:46:51  geoff
@@ -277,6 +286,7 @@ int main (argc, argv)
     static char	outbuf[BUFSIZ];
     int		argno;
     int		arglen;
+    int		i;
 
     Cmd = *argv;
 
@@ -356,9 +366,7 @@ int main (argc, argv)
 	argv = newargv;
 	}
 
-    argv++;
-    argc--;
-    while (argc && **argv == '-')
+    for (argno = 1;  argno < argc  &&  argv[argno][0] == '-';  argno++)
 	{
 	/*
 	 * Trying to add a new flag?  Can't remember what's been used?
@@ -371,8 +379,8 @@ int main (argc, argv)
 	 *	abcdefghijklmnopqrstuvwxyz
 	 *	^^^^^^  ^   ^^^ ^ ^^^ ^^^
 	 */
-	arglen = strlen (*argv);
-	switch ((*argv)[1])
+	arglen = strlen (argv[argno]);
+	switch (argv[argno][1])
 	    {
 	    case 'v':
 		if (arglen > 3)
@@ -384,7 +392,7 @@ int main (argc, argv)
 		      p += 5;
 		    (void) printf ("%s\n", p);
 		    }
-		if ((*argv)[2] == 'v')
+		if (argv[argno][2] == 'v')
 		    {
 		    (void) printf (ISPELL_C_OPTIONS_ARE);
 		    /*
@@ -631,57 +639,34 @@ int main (argc, argv)
 		    preftype = "plain";
 		break;
 	    case 'k':			/* Set keyword tables */
-		p = (*argv) + 2;
-		argv++;
-		argc--;
-		if (argc == 0)
+		/*
+		 * The 'k' keyword can't be processed until after the
+		 * hash header has been read.  For now, we just skip
+		 * over it.
+		 */
+		argno++;
+		if (argno >= argc)
 		    usage ();
-		if (strcmp (p, "texskip1") == 0)
-		    {
-		    if (init_keyword_table (*argv, TEXSKIP1VAR, TEXSKIP1,
-		      0, &texskip1list))
-			usage ();
-		    }
-		else if (strcmp (p, "texskip2") == 0)
-		    {
-		    if (init_keyword_table (*argv, TEXSKIP2VAR, TEXSKIP2,
-		      0, &texskip2list))
-			usage ();
-		    }
-		else if (strcmp (p, "htmlignore") == 0)
-		    {
-		    if (init_keyword_table (*argv, HTMLIGNOREVAR, HTMLIGNORE,
-		      1, &htmlignorelist))
-			usage ();
-		    }
-		else if (strcmp (p, "htmlcheck") == 0)
-		    {
-		    if (init_keyword_table (*argv, HTMLCHECKVAR, HTMLCHECK,
-		      1, &htmlchecklist))
-			usage ();
-		    }
 		break;
 	    case 'T':			/* Set preferred file type */
-		p = (*argv) + 2;
+		p = argv[argno] + 2;
 		if (*p == '\0')
 		    {
-		    argv++;
-		    argc--;
-		    if (argc == 0)
+		    argno++;
+		    if (argno >= argc)
 			usage ();
-		    p = *argv;
+		    p = argv[argno];
 		    }
 		preftype = p;
 		break;
 	    case 'F':			/* Set external deformatting program */
-		p = (*argv) + 2;
+		p = argv[argno] + 2;
 		if (*p == '\0')
 		    {
-		    argv++;
-		    argc--;
-		    if (argc == 0)
+		    argno++;
+		    if (argno >= argc)
 			usage ();
-		    p = *argv;
+		    p = argv[argno];
 		    }
 		defmtpgm = p;
 		tflag = DEFORMAT_NONE;
@@ -710,11 +695,11 @@ int main (argc, argv)
 		if (arglen > 3)
 		    usage ();
 		eflag = 1;
-		if ((*argv)[2] == 'e')
+		if (argv[argno][2] == 'e')
 		    eflag = 2;
-		else if ((*argv)[2] >= '1'  &&  (*argv)[2] <= '5')
-		    eflag = (*argv)[2] - '0';
-		else if ((*argv)[2] != '\0')
+		else if (argv[argno][2] >= '1'  &&  argv[argno][2] <= '5')
+		    eflag = argv[argno][2] - '0';
+		else if (argv[argno][2] != '\0')
 		    usage ();
 		nodictflag++;
 		break;
@@ -737,28 +722,26 @@ int main (argc, argv)
 		break;
 	    case 'f':
 		fflag++;
-		p = (*argv) + 2;
+		p = argv[argno] + 2;
 		if (*p == '\0')
 		    {
-		    argv++;
-		    argc--;
-		    if (argc == 0)
+		    argno++;
+		    if (argno >= argc)
 			usage ();
-		    p = *argv;
+		    p = argv[argno];
 		    }
 		askfilename = p;
 		if (*askfilename == '\0')
 		    askfilename = NULL;
 		break;
 	    case 'L':
-		p = (*argv) + 2;
+		p = argv[argno] + 2;
 		if (*p == '\0')
 		    {
-		    argv++;
-		    argc--;
-		    if (argc == 0)
+		    argno++;
+		    if (argno >= argc)
 			usage ();
-		    p = *argv;
+		    p = argv[argno];
 		    }
 		contextsize = atoi (p);
 		break;
@@ -810,28 +793,24 @@ int main (argc, argv)
 		minimenusize = 2;
 		break;
 	    case 'p':
-		cpd = (*argv) + 2;
+		cpd = argv[argno] + 2;
 		if (*cpd == '\0')
 		    {
-		    argv++;
-		    argc--;
-		    if (argc == 0)
+		    argno++;
+		    if (argno >= argc)
 			usage ();
-		    cpd = *argv;
-		    if (*cpd == '\0')
-			cpd = NULL;
+		    cpd = argv[argno];
 		    }
 		LibDict = NULL;
 		break;
 	    case 'd':
-		p = (*argv) + 2;
+		p = argv[argno] + 2;
 		if (*p == '\0')
 		    {
-		    argv++;
-		    argc--;
-		    if (argc == 0)
+		    argno++;
+		    if (argno >= argc)
 			usage ();
-		    p = *argv;
+		    p = argv[argno];
 		    }
 		if (last_slash (p) != NULL)
 		    (void) strcpy (hashname, p);
@@ -857,44 +836,35 @@ int main (argc, argv)
 		vflag = 1;
 		break;
 	    case 'w':
-		wchars = (*argv) + 2;
+		wchars = argv[argno] + 2;
 		if (*wchars == '\0')
 		    {
-		    argv++;
-		    argc--;
-		    if (argc == 0)
+		    argno++;
+		    if (argno >= argc)
 			usage ();
-		    wchars = *argv;
+		    wchars = argv[argno];
 		    }
 		break;
 	    case 'W':
-		if ((*argv)[2] == '\0')
+		if (argv[argno][2] == '\0')
 		    {
-		    argv++;
-		    argc--;
-		    if (argc == 0)
+		    argno++;
+		    if (argno >= argc)
 			usage ();
-		    minword = atoi (*argv);
+		    minword = atoi(argv[argno]);
 		    }
 		else
-		    minword = atoi (*argv + 2);
+		    minword = atoi (&argv[argno][2]);
 		break;
 	    default:
 		usage ();
 	    }
-	argv++;
-	argc--;
 	}
 
-    if (!argc  &&  !lflag  &&  !aflag   &&  !eflag  &&  !dumpflag)
+    if (argno >= argc  &&  !lflag  &&  !aflag   &&  !eflag  &&  !dumpflag)
 	{
-	if (argc != 0)
-	    usage ();
-	else
-	    {
-	    aflag = 1;
-	    askverbose = 1;
-	    }
+ 	aflag = 1;
+ 	askverbose = 1;
 	}
 
     /*
@@ -902,20 +872,87 @@ int main (argc, argv)
      * the files specified first to see if they exist.  If at least
      * one exists, we continue.
      */
-    for (argno = 0;  argno < argc;  argno++)
+    for (i = argno;  i < argc;  i++)
 	{
-	if (access (argv[argno], R_OK) >= 0)
+	if (access (argv[i], R_OK) >= 0)
 	    break;
 	}
-    if (argno >= argc  &&  !lflag  &&  !aflag  &&  !eflag  &&  !dumpflag)
+    if (i >= argc  &&  !lflag  &&  !aflag  &&  !eflag  &&  !dumpflag)
 	{
 	(void) fprintf (stderr,
-	  argc == 1 ? ISPELL_C_NO_FILE : ISPELL_C_NO_FILES);
+	  argno + 1 == argc ? ISPELL_C_NO_FILE : ISPELL_C_NO_FILES);
 	exit (1);
 	}
+
+    /*
+     * Read the dictionary.
+     */
     if (linit () < 0)
 	exit (1);
 
+    /*
+     * Now process arguments that need the dictionary.  Since we've
+     * rejected invalid switches above, the default case just ignores
+     * Boolean switches.  However, we still need to handle one- and
+     * two-argument switches properly.
+     */
+    for (argno = 1;  argno < argc  &&  argv[argno][0] == '-';  argno++)
+	{
+	arglen = strlen (argv[argno]);
+	switch (argv[argno][1])
+	    {
+	    case 'k':			/* Set keyword tables */
+		/*
+		 * The 'k' keyword can't be processed until after the
+		 * hash header has been read.  For now, we just skip
+		 * over it.
+		 */
+		p = argv[argno] + 2;
+		argno++;
+		if (strcmp (p, "texskip1") == 0)
+		    {
+		    if (init_keyword_table (argv[argno], TEXSKIP1VAR, TEXSKIP1,
+		      0, &texskip1list))
+			usage ();
+		    }
+		else if (strcmp (p, "texskip2") == 0)
+		    {
+		    if (init_keyword_table (argv[argno], TEXSKIP2VAR, TEXSKIP2,
+		      0, &texskip2list))
+			usage ();
+		    }
+		else if (strcmp (p, "htmlignore") == 0)
+		    {
+		    if (init_keyword_table (argv[argno], HTMLIGNOREVAR,
+		      HTMLIGNORE, 1, &htmlignorelist))
+			usage ();
+		    }
+		else if (strcmp (p, "htmlcheck") == 0)
+		    {
+		    if (init_keyword_table (argv[argno], HTMLCHECKVAR,
+		      HTMLCHECK, 1, &htmlchecklist))
+			usage ();
+		    }
+		break;
+	    /*
+	     * Switches that can take cuddled or separate arguments.
+	     */
+	    case 'F':			/* Set external deformatting program */
+	    case 'L':			/* Bottom-of-screen context */
+	    case 'T':			/* Set preferred file type */
+	    case 'W':			/* Specify minimum word length */
+	    case 'd':			/* Specify dictionary */
+	    case 'f':			/* Write results to file */
+	    case 'p':			/* Specify personal dictionary */
+	    case 'w':			/* Specify word characters */
+		if (argv[argno][2] == '\0')
+		    argno++;
+		break;
+	    default:
+		break;
+	    }
+	}
+
     if (preftype == NULL)
 	preftype = getenv (CHARSETVAR);
 
@@ -1010,8 +1047,8 @@ int main (argc, argv)
 
     terminit ();
 
-    while (argc--)
-	dofile (*argv++);
+    for (  ;  argno < argc;  argno++)
+	dofile (argv[argno]);
 
     done (0);
     /* NOTREACHED */
