@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: lookup.c,v 1.54 2020-12-30 22:33:41-08 geoff Exp $";
+    "$Id: lookup.c,v 1.55 2022-02-05 23:23:17-08 geoff Exp $";
 #endif
 
 /*
@@ -46,6 +46,11 @@ static char Rcs_Id[] =
 
 /*
  * $Log: lookup.c,v $
+ * Revision 1.55  2022-02-05 23:23:17-08  geoff
+ * If no default string type is given, use a built-in default instead of
+ * setting chartypes to a null pointer.  Doing so simplifies other code
+ * by not requiring it to do a null check.
+ *
  * Revision 1.54  2020-12-30 22:33:41-08  geoff
  * Recover character-set options from the hash file when loading.
  *
@@ -107,6 +112,17 @@ struct dent *	lookup P ((ichar_t * word, int dotree));
 
 static int	inited = 0;
 
+/*
+ * Default character type if no type is given
+ */
+static struct strchartype deftype =
+    {
+    "default",		/* Name */
+    "nroff",		/* Deformatter */
+    ".txt\0.list\0",	/* Suffixes */
+    RAW_DISPLAY		/* Show string characters as-is */
+    };
+
 int linit ()
     {
     int			hashfd;
@@ -411,7 +427,7 @@ int linit ()
     dumpindex (sflagindex, 0);
 #endif
     if (hashheader.nstrchartype == 0)
-	chartypes = NULL;
+	chartypes = &deftype;
     else
 	{
 	chartypes = (struct strchartype *)
