@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-	"$Id: makedent.c,v 1.60 2005/11/03 22:14:26 geoff Exp $";
+	"$Id: makedent.c,v 1.62 2021-01-09 13:12:35-08 geoff Exp $";
 #endif
 
 /*
@@ -42,7 +42,16 @@ static char Rcs_Id[] =
 
 /*
  * $Log: makedent.c,v $
- * Revision 1.60  2005/11/03 22:14:26  geoff
+ * Revision 1.62  2021-01-09 13:12:35-08  geoff
+ * Rewrite the binary search in stringcharlen to use the new ordering of
+ * stringchars, which uses the character type as the major sort key and
+ * the character itself as the secondary key.
+ *
+ * Revision 1.61  2020-12-30 22:33:41-08  geoff
+ * Fix a binary-search bug that could cause string characters to not be
+ * found in some situations.
+ *
+ * Revision 1.60  2005-11-03 14:14:26-08  geoff
  * When combining capitals, insert new variants at the end of the variant
  * list.  This maintains the order of variants in the personal dictionary.
  *
@@ -788,6 +797,8 @@ int stringcharlen (bufp, canonical)
     int			canonical;	/* NZ if input is in canonical form */
     {
 #ifdef SLOWMULTIPLY
+    static int *        gp[MAXSTRINGCHARS];
+                                        /* Pointers to groupnos */
     static unsigned char *
 			sp[MAXSTRINGCHARS];
     static int		inited = 0;
@@ -806,7 +817,10 @@ int stringcharlen (bufp, canonical)
 	{
 	inited = 1;
 	for (stringno = 0;  stringno < MAXSTRINGCHARS;  stringno++)
+	    {
+	    gp[stringno] = &hashheader.groupnos[stringno];
 	    sp[stringno] = &hashheader.stringchars[stringno][0];
+            }
 	}
 #endif /* SLOWMULTIPLY */
     lowstringno = 0;
@@ -816,45 +830,67 @@ int stringcharlen (bufp, canonical)
 	{
 	stringno = (lowstringno + highstringno) >> 1;
 #ifdef SLOWMULTIPLY
+        if (dupwanted < gp[stringno])
+            {
+            highstringno = stringno - 1;
+            continue;
+            }
+        else if (dupwanted > gp[stringno])
+            {
+            lowstringno = stringno + 1;
+            continue;
+            }
 	stringcur = sp[stringno];
 #else /* SLOWMULTIPLY */
+        if (dupwanted < hashheader.groupnos[stringno])
+            {
+            highstringno = stringno - 1;
+            continue;
+            }
+        else if (dupwanted > hashheader.groupnos[stringno])
+            {
+            lowstringno = stringno + 1;
+            continue;
+            }
 	stringcur = &hashheader.stringchars[stringno][0];
 #endif /* SLOWMULTIPLY */
 	bufcur = bufp;
-	while (*stringcur)
-	    {
-	    if (*bufcur++ != *stringcur)
-		break;
-	    /*
-	    ** We can't use autoincrement above because of the
-	    ** test below.
-	    */
-	    stringcur++;
-	    }
+        /*
+         * Doing a correct comparison is a touch tricky because the
+         * data in bufp isn't null-terminated; we have to identify the
+         * stringchar as a prefix in bufp.  We don't bother to check
+         * for a null byte in bufp because a null byte will just show
+         * up as a mismatch with the current stringchar.
+         */
+        while (*stringcur != '\0'  &&  *bufcur == *stringcur)
+            {
+            stringcur++;
+            bufcur++;
+            }
+        /*
+         * If we got to the end of the stringchar, we have a match
+         * regardless of what the next character in bufcur is.  But
+         * it's a "true" match only if dupwanted also matches.
+         * Otherwise we need to continue the search.
+         */
 	if (*stringcur == '\0')
-	    {
-	    if (hashheader.groupnos[stringno] == dupwanted)
-		{
-		/* We have a match */
-		laststringch = hashheader.stringdups[stringno];
+            {
+            laststringch = hashheader.stringdups[stringno];
 #ifdef SLOWMULTIPLY
-		return stringcur - sp[stringno];
+            return stringcur - sp[stringno];
 #else /* SLOWMULTIPLY */
-		return stringcur - &hashheader.stringchars[stringno][0];
+            return stringcur - &hashheader.stringchars[stringno][0];
 #endif /* SLOWMULTIPLY */
-		}
-	    else
-		--stringcur;
-	    }
-	/* No match - choose which side to search on */
-	if (*--bufcur < *stringcur)
-	    highstringno = stringno - 1;
-	else if (*bufcur > *stringcur)
-	    lowstringno = stringno + 1;
-	else if (dupwanted < hashheader.groupnos[stringno])
-	    highstringno = stringno - 1;
-	else
-	    lowstringno = stringno + 1;
+            }
+        /*
+         * We aren't at the end of the stringchar, so we can decide
+         * our direction based on the mismatch between the stringchar
+         * and bufp.
+         */
+        if (*bufcur < *stringcur)
+            highstringno = stringno - 1;
+        else
+            lowstringno = stringno + 1;
 	}
     laststringch = -1;
     return 0;			/* Not a string character */
