@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: term.c,v 1.55 2015-02-08 00:35:41-08 geoff Exp $";
+    "$Id: term.c,v 1.57 2020-12-30 22:33:41-08 geoff Exp $";
 #endif
 
 /*
@@ -46,6 +46,15 @@ static char Rcs_Id[] =
 
 /*
  * $Log: term.c,v $
+ * Revision 1.57  2020-12-30 22:33:41-08  geoff
+ * Get rid of an undeclared-function warning.  This might not be
+ * backward-compatible to very old systems.
+ *
+ * Revision 1.56  2020-10-26 11:39:55-07  geoff
+ * Include header files needed for C99 function declarations.  This probably
+ * breaks backwards compatibility but I don't have time to research it and
+ * add appropriate config.X options.  Sigh.
+ *
  * Revision 1.55  2015-02-08 00:35:41-08  geoff
  * Add POSIX termios support (thanks to Christian Weisgerber for the patch).
  *
@@ -99,6 +108,8 @@ static char Rcs_Id[] =
 #endif
 #endif
 #include <signal.h>
+#include <sys/ioctl.h>
+#include <sys/wait.h>
 
 void		ierase P ((void));
 void		imove P ((int row, int col));
